import imp
import os
import sys


class Finder(object):
  INIT_DOT_PY = '__init__%spy' % os.extsep

  def __init__(self, prefix, transform):
    self._prefix = prefix.split('.')
    self._transform = transform

  def find_module(self, name, path=None):
    components = name.split('.')
    it = iter(components)
    if not any(c in it for c in self._prefix):
      return None
    subname = components[-1]
    file, path, description = imp.find_module(subname, path)
    if description[-1] == imp.PKG_DIRECTORY:
      filename = os.path.join(path, self.INIT_DOT_PY)
      file = open(filename)
    elif file is None:
      return None
    else:
      filename = path
    return Loader(file, path, filename, self._transform)


class Loader(object):
  def __init__(self, file, path, filename, transform):
    self._file = file
    self._path = path
    self._filename = filename
    self._transform = transform

  def is_package(self, mod_name):
    return len(self._filename) > len(self._path)

  def compile(self, name):
    filename = self._filename
    source = self._file.read()
    self._file.close()
    return compile(self._transform(source, name, filename), filename, 'exec')

  def prepare_module(self, module):
    filename, path = self._filename, self._path
    module.__file__ = filename
    module.__loader__ = self
    if len(filename) > len(path):
      module.__path__ = [path]
      module.__package__ = module.__name__
    else:
      module.__package__ = module.__name__.rpartition('.')[0]
    return module

  def load_module(self, name):
    imp.acquire_lock()
    try:
      try:
        module = sys.modules[name]
      except KeyError:
        module = imp.new_module(name)
        sys.modules[name] = module
      self.prepare_module(module)
      code = self.compile(name)
      try:
        exec code in module.__dict__
        return module
      except:
        del sys.modules[name]
        raise
    finally:
      imp.release_lock()
      self._file.close()
