from __future__ import print_function

import ast
import unittest
import sys

import context
from my2to3.hook import Finder
from my2to3.visit import DepthVisitor
from my2to3.visit import InstrumentVisitor

import my2to3.runtime


class BinOpVisitor(InstrumentVisitor):
  def visit_BinOp(self, node):
    self.visit_children(node)
    if isinstance(node.op, ast.Div):
      node.left = self.make_trace(node.left)
      node.right = self.make_trace(node.right)
    return node


class TestVisitBinOp(unittest.TestCase):
  def setUp(self):
    self._calls = []
    self._finder = None

  def transform(self, source, name, filename):
    t = ast.parse(source, filename)
    t = DepthVisitor()(t)
    t = BinOpVisitor(name)(t)
    return t

  def info(self, data):
    self._calls.append(data)

  def test_visit_sample(self):
    import sys
    print(sys.argv)
    self._finder = Finder('sample', self.transform)
    sys.meta_path.append(self._finder)

    my2to3.runtime.logger = self

    import sample.sample

    self.assertEqual(len(self._calls), 2)
    self.assertEqual(
      self._calls,
      [
      '{"loc": {"depth": 3, "col": 4, "line": 5, "name": "sample.sample"}, "type": "<type \'int\'>", "value": "2", "desc": null}',
      '{"loc": {"depth": 3, "col": 8, "line": 5, "name": "sample.sample"}, "type": "<type \'int\'>", "value": "3", "desc": null}'
      ]
    )

  def tearDown(self):
    del my2to3.runtime.logger
    sys.meta_path.remove(self._finder)
    del sys.modules['sample']
    del sys.modules['sample.sample']
